#include  <stdio.h>
#include  <stdlib.h>

#include  <tthimage.h>

/* ---------------------------------------------------------------- */
#include  "tripoteur.h"

/* ---------------------------------------------------------------- */

int do_tripoteur(Image_Desc *src, Image_Desc *dst, int k)
{
int		x, y;

for (y=0; y<src->height; y++) {
	for (x=0; x<src->width; x++) {
		dst->Rpix[y][x] = src->Gpix[y&0xff8][x] & 0xc0;
		dst->Gpix[y][x] = src->Bpix[y][x]       & 0xc0;
		dst->Bpix[y][x] = src->Rpix[y][x&0xff8] & 0xc0;
		}
	}

return 666;
}

/* ---------------------------------------------------------------- */

int tripoteur(char *srcname, char *dstname)
{
Image_Desc	*src, *dst;
int		foo;

#if DEBUG_LEVEL
fprintf(stderr, "tripote '%s' -> '%s'\n", srcname, dstname);
#endif

if (NULL==(src=Image_TGA_alloc_load(srcname))) {
	perror("chargement image");
	exit(2);
	}
	
if (NULL==(dst=Image_clone(src, 0))) {
	perror("no memory");
	exit(3);
	}

foo = do_tripoteur(src, dst, 0);

foo = Image_TGA_save(dstname, dst, 0);
#if DEBUG_LEVEL
fprintf(stderr, "sauve -> %d\n", foo);
#endif

return 0;
}

/* ---------------------------------------------------------------- */
int do_distordeur(Image_Desc *src, Image_Desc *map, Image_Desc *dst,
                                      float fk, int deltaX, int deltaY)
{
int             xs, ys, xd, yd, xm, ym;
float           coefXm, coefYm, coefXs, coefYs, offx, offy;
int             rm, gm, bm, foo;
RGBA		rgba;

float lutX[256], lutY[256];


if (fk == 0.0)
        {
        fprintf(stderr, "%s : can't divide by zero\n", __func__);
        return DIVISOR_IS_ZERO;
        }

for (foo=0; foo<256; foo++) {
	lutX[foo] = (float)(foo + deltaX) * fk;
	lutY[foo] = (float)(foo + deltaY) * fk;
#if DEBUG_LEVEL
	printf("luts %f %f\n", lutX[foo], lutY[foo]);
#endif
	}

fprintf(stderr, "%s : coef is %12.8f\n", __func__, fk);
fprintf(stderr, "                deltas     %d %d\n", deltaX, deltaY);

coefXm = (float)dst->width  / (float)map->width;
coefYm = (float)dst->height / (float)map->height;

coefXs = (float)dst->width  / (float)src->width;
coefYs = (float)dst->height / (float)src->height;

#if DEBUG_LEVEL 
fprintf(stderr, "    src  %4d %4d\n", src->width, src->height);
fprintf(stderr, "    map  %4d %4d\n", map->width, map->height);
fprintf(stderr, "    dst  %4d %4d\n", dst->width, dst->height);
fprintf(stderr, "coefs map %f %f\n", coefXm, coefYm);
fprintf(stderr, "coefs src %f %f\n", coefXs, coefYs);
#endif

Image_default_RGBA(&rgba, "border color", 0);

for (yd=0; yd<dst->height; yd++) {
        for (xd=0; xd<dst->width; xd++) {

                xm = (int)((float)xd / coefXm);
                ym = (int)((float)yd / coefYm);

		rm = map->Rpix[ym][xm];
		gm = map->Gpix[ym][xm];
		bm = map->Bpix[ym][xm];
				
                offx = (float)xd + lutX[rm];
                offy = (float)yd + lutY[bm];
		
                xs = (int)(offx/coefXs);
                ys = (int)(offy/coefYs);

                if ( (xs>=0) && (xs<src->width) &&
                     (ys>=0) && (ys<src->height) ) {
			Image_getpix_bilin(src, (offx/coefXs), (offy/coefYs),
						&rm, &gm, &bm);                        
			}
		else	{
			rm = rgba.r;
			gm = rgba.g;
			bm = rgba.b;
			}
			
		dst->Rpix[yd][xd] = rm;
		dst->Gpix[yd][xd] = gm;
		dst->Bpix[yd][xd] = bm;
		
                }
#if DEBUG_LEVEL > 1
        fprintf(stderr, "%s : %4d / %4d\r", __func__, yd, dst->height);
#endif
        }
        
return 0;
}

/* ---------------------------------------------------------------- */
int deformeur(char *srcname, char *mapname, char *dstname)
{
Image_Desc	*src, *map, *dst;
int		foo;

#if DEBUG_LEVEL
fprintf(stderr, "%s : '%s' & '%s' -> '%s'\n", __func__, 
			srcname, mapname, dstname);
#endif

if (NULL==(src=Image_TGA_alloc_load(srcname))) {
	perror("chargement image");
	exit(2);
	}
Image_kill_alpha_channel(src);

/* verifier que les dimensions de l'image son paires, 
   parce que sinon, avconv refuse de bosser */
if ( (src->width & 1) || (src->height & 1) ) {
	fprintf(stderr, "dimension image '%s' impaire.\n", srcname);
	exit(1);
	}
	
if (NULL==(map=Image_TGA_alloc_load(mapname))) {
	perror("chargement map");
	exit(2);
	}
Image_kill_alpha_channel(map);
		
if (NULL==(dst=Image_clone(src, 0))) {
	perror("no memory");
	exit(3);
	}

foo = do_distordeur(src, map, dst, 0.38, -120, -150);
if (foo) {
	fprintf(stderr, "hu ho %d ?\n", foo);
	}
	
foo = Image_TGA_save(dstname, dst, 0);
#if DEBUG_LEVEL
fprintf(stderr, "sauve -> %d\n", foo);
#endif

Image_DeAllocate(src);	free(src);
Image_DeAllocate(map);	free(map);
Image_DeAllocate(dst);	free(dst);

return 0;
}

/* ---------------------------------------------------------------- */
